
/*****************************************************************
    INCLUDE FILES
*****************************************************************/   

#include "shapes.inc"
#include "screen.inc"
#include "iso_csg.inc"
#include "colors.inc"

/*****************************************************************
    GLOBALS
*****************************************************************/   

#default { finish { ambient 0 } }
#declare S = seed(10000100101010101);

/*****************************************************************
    GLOBAL SETTINGS
*****************************************************************/    

global_settings {
	assumed_gamma 1.0
	max_trace_level 100
}

/*****************************************************************
    START OF PROCESSING
*****************************************************************/    

#debug "\nStart of Process\n"


#if ( on )

	#declare UseIso = off;
	#declare MG = 4.0;

	#declare Finish_Stone =
	finish {
		ambient 0
		diffuse 1
		specular 1
		roughness 0.02
		reflection 0
	};

	#macro Decl_Pf_Crackle (Scale)
		//define a crackle pattern as a function with scaling and translation applied
		#undef Pf_Crackle
		#declare Pf_Crackle =
		function {
			pigment {
				crackle
				poly_wave 1
				form <1,0,0>
				scale Scale
			}
		};
	#end

	#macro Decl_Pf_Boxed (Scale)
		//define a boxed pattern as a function with scaling and translation applied
		#undef Pf_Boxed
		#declare Pf_Boxed =
		function {
			pigment {
				boxed
				scale Scale*1.075
			}
		};
	#end

	#macro Decl_Pf_Edges (Contrast)
		//define a function which scales smaller toward the edges of the stone
		//here a predefined crackle function nested inside the
		//predefined boxed function

		#undef Pf_Edges
		#declare Pf_Edges =
		function {
			pigment {
				function { Pf_Boxed(x,y,z).gray }
				pigment_map {
					[ 0.00
						function { (1-Contrast)+Pf_Crackle(x,y,z).gray*Contrast }
						scale 0.08
					]
					[ 0.12
						function { (1-Contrast/2)+Pf_Crackle(x,y,z).gray*Contrast/2 }
						scale 0.16
					]
					[ 0.36
						function { (1-Contrast/3)+Pf_Crackle(x,y,z).gray*Contrast/3 }
						scale 0.24
					]
					[ 1.00
						function { (1-Contrast/4)+Pf_Crackle(x,y,z).gray*Contrast/4 }
						scale 0.32
					]
				}
			}
		};
	#end

	#macro Decl_Pf_Granite (Scale)
		//define a granite pattern as a function with scaling and translation applied
		#undef Pf_Granite
		#declare Pf_Granite =
		function {
			pigment {
				granite
				scale Scale
				translate rand(S)*100
			}
		};
	#end

	#macro Decl_P_Stone (Pf,PMap)
		//define a granite pigment using a predefined granite function
		#declare P_Stone =
		pigment {
			function { Pf_Granite(x,y,z).gray }
			pigment_map {
				PMap
			}
		};
	#end

	#macro Decl_Pf_Bumps (Scale)
		//define a bump pattern as a function with scaling and translation applied
		#undef Pf_Bumps
		#declare Pf_Bumps =
		function {
			pigment {
				bumps
				scale Scale
				translate <100*rand(S),100*rand(S),0>
			}
		};
	#end

	#macro Decl_Pf_GradientY (Height)
		//define a gradient pattern as a function with scaling and translation applied
		#undef Pf_GradientY
		#declare Pf_GradientY =
		function {
			pigment {
				gradient <0,1,0>
				scale <1,2*Height,1>
				translate <0,Height,0>
			}
		};
	#end

	#macro Decl_P_Stain ( StainColor, StonePigment )
		//Apply dripping stains to underlying stone pigment.  Result is a pigment

		#ifdef ( Pf_Edges )
			#declare P_Base =
			pigment {
				function { Pf_Edges(x,y,z).gray }
				pigment_map {
					[ 0 rgb StainColor ]
					[ 1 StonePigment ]
				}
			};
		#else
			#declare P_Base = StonePigment;
		#end

		Decl_Pf_Bumps ( <.075,1.5,1> )

		#declare P_Stain_Top =
		pigment {
			function { Pf_GradientY(x,y,z).gray }
			pigment_map {
				[ 0.0
					P_Base
				]
				[ 0.9
					function { Pf_Bumps(x,y,z).gray }
					pigment_map {
						[ 0
							rgb StainColor
						]
						[ 1
							P_Base
						]
					}
				]
			}
		};

		#declare P_Stain_Bottom =
		pigment {
			function { Pf_GradientY(x,y,z).gray }
			pigment_map {
				[ 0
					average
					pigment_map {
						#local M=0;
						#while(M<5)
							Decl_Pf_Bumps ( <.25,0.5,1> )
							[ 1
								function { Pf_Bumps(x,y,z).gray }
								pigment_map {
									[ 0  rgb StainColor ]
									[ 1  P_Base ]
								}
							]
							#local M=M+1;
						#end
					}
				]
				[ 0.5
					P_Base
				]
			}
		};

		#declare P_Stain =
		pigment {
			function { Pf_GradientY(x,y,z).gray }
			pigment_map {
				[ 0.00  rgb StainColor ]
				[ 0.01  P_Stain_Bottom ]
				[ 0.99  P_Stain_Top ]
				[ 1.00  rgb StainColor ]
			}
		};
	#end

	#macro Create_Stone ( Width, Height, Thick, Round, StainColor )

		//produce the needed function definitions
		Decl_Pf_Granite ( 0.075 )        //scale 0.075
		Decl_Pf_Crackle ( 0.75 )         //scale 0.75
		Decl_Pf_Boxed ( <Width,Height,1> )
		Decl_Pf_Edges ( 1 )               //Contrast = 1  1 = full
		Decl_Pf_GradientY ( Height )

		//probably not the most logical place for this but:
		#local StonePigmentMap =
		pigment_map {
			[ 0 rgb CHSL2RGB(<24,0.1,0.99>) ]
			[ 1 rgb CHSL2RGB(<24,0.1,0.1>) ]
		};

		//produce the pigment definitions
		Decl_P_Stone (Pf_Granite,StonePigmentMap)
		Decl_P_Stain ( StainColor, P_Stone )

		#declare Stone =
		#if ( UseIso )
			#undef Fn_Stone
			#declare Fn_Stone =
			//IC_Box ( <-Width+0.01,-Height+0.01,-Thick+0.01>, <Width-0.01,Height-0.01,Thick-0.01> );
			//IC_Round_Box ( <-Width,-Height,-Thick>, <Width,Height,Thick>, 0.03 );
			IC_Scale (IC_Superellipsoid (0.05, 0.05), <Width,Height,Thick>)

			isosurface {
				function {
					Fn_Stone (x,y,z)
					#ifdef ( Pf_Granite )
						- min ( 0.003,Pf_Granite(x,y,z).gray*0.005 )
					#end
					#ifdef ( Pf_Boxed )
						- Pf_Boxed(x,y,z).gray*0.002
					#end
				}
				contained_by {
					box {
						<-Width,-Height,-Thick*2>, <Width,Height,Thick>
						//< -Width-0.5,-Height-0.5,-Thick-0.5>, <Width+0.5,Height+0.5,Thick+0.5>
					}
				}
				accuracy 0.01
				max_gradient MG

				texture {
					pigment {
						P_Stain
					}
					finish {
						Finish_Stone
					}
				}
			};

		#else
			object {
				Round_Box_Merge ( <-Width,-Height,-Thick>, <Width,Height,Thick>, Round )
				//box { <-Width+0.01,-Height+0.01,-Thick+0.01>, <Width-0.01,Height-0.01,Thick-0.01> }

				texture {                        
					pigment {                
						P_Stain
						//P_Base
					}
					finish {                 
						Finish_Stone
					}
					normal {
						pigment_pattern {
							P_Base
						}                      //this is awkward, P_Base is
						bump_size 0.02         //defined in the Decl_P_Stain macro
					}
				}
			};
		#end

	#end

	#if ( off ) //test one brick

		#local Width    = 1.5;
		#local Height   = 1.0;
		#local Thick    = 0.25;
		#local Round    = 0.03;
		#local StainColor = <0,0,0>;

		Create_Stone ( Width, Height, Thick, Round, StainColor )

		object {
			Stone
			texture {
				pigment {
					P_Stain
				}
				finish {
					Finish_Stone
				}
				normal {
					pigment_pattern { P_Base }
				}
			}
		}

	#end

	#if ( on ) //create a wall of bricks

		#local I=-1;
		#while(I<3)
			#local H=-2;
			#while(H<3)
				#if( I<2 & I>-2 & H<2 & H>-2 )
					#local K=0;
					#while(K<2)
						#local Width    = 1.5;
						#local Height   = 1.0;
						#local Thick    = 0.25;
						#local Round    = 0.04; 
						#local StainColor = <0,0,0>; 

						Create_Stone ( Width, Height, Thick, Round, StainColor )

						object {
							Stone
							translate <-1.505+K*3,0.005+I*2,0>
						}

						#local K=K+1;
					#end
				#else
					#local Width    = 1.0;
					#local Height   = 1.0;
					#local Thick    = 0.25;
					#local Round    = 0.03;
					#local StainColor = <0,0,0>;   //rand(S);

					Create_Stone ( Width, Height, Thick, Round, StainColor)

					object {
						Stone
						translate <0.005+H*2,0.005+I*2,0>
					}

				#end
				#local H=H+1;
			#end
			#local I=I+1;
		#end

	#end  

#end   

#debug "\nEnd of Process\n"  

/*****************************************************************
    CAMERA LIGHTS
*****************************************************************/

#if ( on )

	#declare CamPos = vrotate ( <0,0,-60>, <20,40,0> )+<0,0,0>;
	#declare LookAt = vrotate ( <0,0, 0>, <0,0,0> )+<0,0,0>;
	#declare LensAngle =  10;

	Set_Camera( CamPos, LookAt, LensAngle )

	#local UseAreaLight=off;
	#local UseSpotLight=off;
	#local Size = 1000;
	#local Brightness = 1.50;

	light_source {
		<0,0,0>

		rgb 1*Brightness

		#if ( UseSpotLight )
			spotlight
		#end

		translate vrotate ( <0,0,-Size*0.95>, <60,-70,0> )

		#if ( UseSpotLight )
			point_at <0,3,0>
			radius 0.1
			tightness 2
			falloff 0.5
		#end

		#if ( UseAreaLight )
			area_light
			<Size*0.05,0,0>, <0,0,Size*0.05>,
			33, 33
			adaptive 1
			//jitter
			circular
			orient

		#end
	}

#end

